<?php

namespace Drupal\Tests\akismet\Functional;

use Drupal\Core\Logger\RfcLogLevel;
use Drupal\user\RoleInterface;

/**
 * Tests module installation and key error handling.
 *
 * @group akismet
 */
class InstallationTest extends AkismetTestBase {

  /**
   * Modules to enable.
   *
   * @var string[]
   */
  public static $modules = [
    'dblog',
    'node',
    'comment',
  ];

  /**
   * Use local mock server.
   *
   * @var bool
   */
  protected $useLocal = TRUE;

  /**
   * Disable the default set up.
   *
   * @var bool
   */
  public $disableDefaultSetup = TRUE;

  /**
   * Do not create keys in the base setup.
   *
   * @var bool
   */
  protected $createKeys = FALSE;

  /**
   * Do not set up Akismet.
   *
   * @var bool
   */
  protected $setupAkismet = FALSE;

  /**
   * User to run tests with.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $webUser = NULL;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $this->adminUser = $this->drupalCreateUser([
      'access administration pages',
      'administer site configuration',
      'administer modules',
      'administer permissions',
    ]);
    $this->webUser = $this->drupalCreateUser([]);
  }

  /**
   * Tests status handling after installation.
   *
   * We walk through a regular installation of the Akismet module instead of
   * using setUp() to ensure that everything works as expected.
   *
   * Note: Partial error messages tested here; hence, no t().
   */
  public function testInstallationProcess() {
    $message_short_invalid = t('The configured Akismet API key is invalid.');
    $message_invalid = t('The API key could not be verified.');
    $message_valid = t('Akismet verified your key. The service is operating correctly.');
    $message_missing = t('The Akismet API key is not configured yet.');
    $message_server = t('The Akismet server could not be contacted. Please make sure that your web server can make outgoing HTTP requests.');
    $message_saved = t('The configuration options have been saved.');
    $admin_message = t('Visit the Akismet settings page to add your API key.');
    $install_message = t('Akismet installed successfully. Visit the @link to add your API key.', [
      '@link' => t('Akismet settings page'),
    ]);

    $this->drupalLogin($this->adminUser);

    // Ensure there is no requirements error by default.
    $this->drupalGet('admin/reports/status');
    $this->clickLink('Run cron');

    // Install the Akismet module.
    $this->drupalPostForm('admin/modules', ['modules[akismet][enable]' => TRUE], t('Install'));
    $this->assertText($install_message);

    // Now we can add the test module for the rest of the form tests.
    \Drupal::service('module_installer')->install([
      'akismet_test',
      'akismet_test_server',
    ]);
    $settings = \Drupal::configFactory()->getEditable('akismet.settings');
    $settings->set('log_level', RfcLogLevel::DEBUG);
    $settings->save();

    $this->resetAll();

    // Verify that forms can be submitted without valid Akismet module
    // configuration.
    $this->drupalLogin($this->webUser);
    $edit = [
      'title' => 'spam',
    ];
    $this->drupalPostForm('akismet-test/form', $edit, t('Save'));
    $this->assertText('Successful form submission.');

    // Assign the 'administer akismet' permission and log in a user.
    $this->drupalLogin($this->adminUser);
    $edit = [
      RoleInterface::AUTHENTICATED_ID . '[administer akismet]' => TRUE,
    ];
    $this->drupalPostForm('admin/people/permissions', $edit, t('Save permissions'));

    // Verify presence of 'empty keys' error message.
    $this->drupalGet('admin/config/content/akismet');
    $this->assertText($message_missing);
    $this->assertNoText($message_invalid);

    // Verify requirements error about missing API keys.
    $this->drupalGet('admin/reports/status');
    $this->assertText($message_missing);
    $this->assertText($admin_message);

    // Configure invalid keys.
    $edit = [
      'api_key' => 'the-invalid-akismet-api-key-value',
    ];
    $this->drupalGet('admin/config/content/akismet/settings');
    $this->drupalPostForm(NULL, $edit, t('Save configuration'), ['watchdog' => RfcLogLevel::EMERGENCY]);
    $this->assertText($message_saved);
    $this->assertNoText(self::FALLBACK_MESSAGE);

    // Verify presence of 'incorrect keys' error message.
    $this->assertText($message_short_invalid);
    $this->assertNoText($message_missing);

    // Verify requirements error about invalid API keys.
    $this->drupalGet('admin/reports/status', ['watchdog' => RfcLogLevel::EMERGENCY]);
    $this->assertText($message_short_invalid);

    // Ensure unreachable servers.
    \Drupal::state()->set('akismet.testing_use_local_invalid', TRUE);

    // Verify presence of 'network error' message.
    $this->drupalGet('admin/config/content/akismet/settings', ['watchdog' => RfcLogLevel::EMERGENCY]);
    $this->assertText($message_server);
    $this->assertNoText($message_missing);
    $this->assertNoText($message_invalid);

    // Verify requirements error about network error.
    $this->drupalGet('admin/reports/status', ['watchdog' => RfcLogLevel::EMERGENCY]);
    $this->assertText($message_server);
    $this->assertNoText(self::FALLBACK_MESSAGE);

    // From here on out the watchdog errors are just a nuisance.
    $this->assertWatchdogErrors = FALSE;

    // Prepare for using the valid test client.
    \Drupal::state()->setMultiple([
      'akismet.testing_use_local' => TRUE,
      'akismet.testing_use_local_invalid' => FALSE,
    ]);
    $this->resetAll();

    $this->getClient(TRUE);

    $edit = [
      'api_key' => 'validkey',
    ];
    $this->drupalPostForm('admin/config/content/akismet/settings', $edit, t('Save configuration'));
    $this->assertText($message_saved);
    $this->assertText($message_valid);
    $this->assertNoText($message_missing);
    $this->assertNoText($message_invalid);

    // Verify that deleting keys throws the correct error message again.
    $this->drupalGet('admin/config/content/akismet/settings');
    $this->assertText($message_valid);
    $edit = [
      'api_key' => '',
    ];
    $this->drupalPostForm(NULL, $edit, t('Save configuration'));
    $this->assertText($message_saved);
    // The module incorrectly shows this message in this condition. See
    // https://www.drupal.org/project/akismet/issues/3210899
    // $this->assertNoText($message_valid);
    $this->assertText($message_missing);
    $this->assertNoText($message_invalid);
  }

}
